@@ -66,6 +66,7 @@ document.addEventListener('DOMContentLoaded', function() {
     if (ocsp.status === "good") return '<span class="badge badge-ok">Good</span>';
     if (ocsp.status === "revoked") return '<span class="badge badge-expired">Revoked</span>';
     if (ocsp.status === "unknown") return '<span class="badge badge-warning">Unknown</span>';
+    if (ocsp.status === "no_ocsp_url") return '<span class="badge badge-warning">NOT DEFINED</span>';
     // Consider other statuses like 'error' or specific error messages if available in your data
     return '<span class="badge badge-expired">Error</span>'; // Default for other cases like error
   }
@@ -115,7 +116,7 @@ document.addEventListener('DOMContentLoaded', function() {
       summaryTableBody.innerHTML += `
         <tr>
           <td data-bs-toggle="tooltip" data-bs-title="The domain name that was analyzed."><b>${r.domain || '-'}</b></td>
-          <td data-bs-toggle="tooltip" data-bs-title="The overall status of the TLS analysis for this domain.">${statusBadge(r.status)}</td>
+          <td data-bs-toggle="tooltip" data-bs-title="${r.connection_health && r.connection_health.error ? r.connection_health.error : 'The overall status of the TLS analysis for this domain.'}">${statusBadge(r.status)}</td>
           <td data-bs-toggle="tooltip" data-bs-title="Common Name (CN) of the leaf certificate.">${leaf.common_name || '-'}</td>
           <td data-bs-toggle="tooltip" data-bs-title="Expiration date of the leaf certificate and days remaining.">${leaf.not_after ? (leaf.not_after.substring(0, 10)) : '-' }<br>
               ${expiryBadge(leaf.days_remaining ?? 0)}
@@ -150,6 +151,7 @@ document.addEventListener('DOMContentLoaded', function() {
     const leaf = (result.certificates || []).find(c => c.chain_index === 0 && !c.error) || {};
     const transparencyDetailsHtml = renderTransparencyDetailsTable(result.transparency, idx);
     const caaDetailsHtml = renderCaaDetails(result.caa_check, idx);
+    const connectionError = result.connection_health && result.connection_health.error;
     // Quick info
     // Determine CSS class for expiry text/badge based on days remaining
     // expired: Certificate has expired (days < 0)
@@ -194,6 +196,7 @@ document.addEventListener('DOMContentLoaded', function() {
         </div>
         <!-- Alerts -->
         ${result.error_message ? `<div class="alert alert-danger mt-2"><b>Overall Status:</b> ${result.error_message}</div>` : ""}
+        ${(connectionError && connectionError !== result.error_message) ? `<div class="alert alert-danger mt-2"><b>Connection:</b> ${connectionError}</div>` : ""}
         <!-- Accordion details -->
         <div class="accordion mt-4" id="accordion-${idx}">
           <div class="accordion-item">
@@ -406,6 +409,7 @@ document.addEventListener('DOMContentLoaded', function() {
     const insecure = formData.get('insecure') === 'true';
     const noTransparency = formData.get('no_transparency') === 'true';
     const noCrlCheck = formData.get('no_crl_check') === 'true';
+    const noOcspCheck = formData.get('no_ocsp_check') === 'true';
     const noCaaCheck = formData.get('no_caa_check') === 'true';
 
     const payload = {
@@ -414,6 +418,7 @@ document.addEventListener('DOMContentLoaded', function() {
       insecure: insecure,
       no_transparency: noTransparency,
       no_crl_check: noCrlCheck,
+      no_ocsp_check: noOcspCheck,
       no_caa_check: noCaaCheck
     };
 
@@ -470,13 +475,17 @@ document.addEventListener('DOMContentLoaded', function() {
                 statusText = '❓ Unknown';
                 ocspStatusEl.className = 'text-secondary';
                 break;
+              case 'no_ocsp_url':
+                statusText = 'NOT DEFINED';
+                ocspStatusEl.className = 'text-warning';
+                break;
               default:
                 statusText = '❌ Error';
                 ocspStatusEl.className = 'text-danger';
             }
             ocspStatusEl.textContent = statusText;
-            ocspUrlEl.textContent    = (ocsp.details && ocsp.details.checked_url) || '';
-            ocspDetailEl.textContent = (ocsp.details && (ocsp.details.revocation_reason || ocsp.details.error)) || '';
+            ocspUrlEl.textContent    = ocsp.checked_url || '';
+            ocspDetailEl.textContent = (ocsp.details && (ocsp.details.revocation_reason || ocsp.details.error || ocsp.details.message)) || '';
           }
         }
       } else {
